package metrics;

import java.util.HashMap;

import javax.swing.JFormattedTextField;
import javax.swing.JFrame;
import javax.swing.JPanel;

import testapp.Dictionary;
import testapp.Dictionary.Bundle;
import testapp.JOptionDialog;
import testapp.tapedpane.JTappedPane_Panel;

public class DynamicalPartialMetric extends DistanceMetric {

	int limit = 10;
	double norm = 2.0;

	protected DynamicalPartialMetric(HashMap<String, Object> map) {
		super(map);
		initParameter(map);
	}

	@Override
	protected double innerCalculate(double[] input1, double[] input2) {
		Double sum = 0D;
		double[] mem = new double[limit];
		for (int i = 0; i < limit; i++) {
			mem[i] = Double.MAX_VALUE;
		}
		for (int i = 0; i < input1.length; i++) {
			double delta = Math.abs(input1[i] - input2[i]);
			if (delta < mem[limit - 1]) {
				int j = limit - 1;
				while (j > 0 && mem[j - 1] > delta)
					mem[j] = mem[--j];
				mem[j] = delta;
			}
		}
		for (int i = 0; i < limit; i++)
			sum += Math.pow(mem[i], norm);
		return Math.pow(sum, 1 / norm);
	}

	@Override
	protected double innerCalculate(int[] input1, int[] input2) {
		Double sum = 0D;
		int[] mem = new int[limit];
		for (int i = 0; i < limit; i++) {
			mem[i] = Integer.MAX_VALUE;
		}
		for (int i = 0; i < input1.length; i++) {
			int delta = Math.abs(input1[i] - input2[i]);
			if (delta < mem[limit - 1]) {
				int j = limit - 1;
				while (j > 0 && mem[j - 1] > delta) {
					mem[j] = mem[--j];
				}
				mem[j] = delta;
			}
		}
		for (int i = 0; i < limit; i++)
			sum += Math.pow(mem[i], norm);
		return Math.pow(sum, 1 / norm);
	}

	@Override
	public String getName() {
		return Dictionary.get(Bundle.METRIC, "dynamicalpartial");
	}

	@Override
	public String toString() {
		return getName() + ":" + Dictionary.get(Bundle.METRIC, "quadraticnorm")
				+ "=" + norm + " / "
				+ Dictionary.get(Bundle.METRIC, "quadraticlimit") + "=" + limit;
	}

	@Override
	public JOptionDialog getOptionsDialog(JFrame parent) {
		return new DynamicalPartialOptionsDialog(parent, this);
	}

	private class DynamicalPartialOptionsDialog extends JOptionDialog {

		public DynamicalPartialOptionsDialog(JFrame parent,
				DistanceMetric metric) {
			super(parent, metric.getName());
		}

		/**
		 * 
		 */
		private static final long serialVersionUID = 1L;

		JTappedPane_Panel northPanel;
		JFormattedTextField fieldNorm;
		JFormattedTextField fieldLimit;

		@Override
		public JPanel getNorthPanel() {
			northPanel = new JTappedPane_Panel(null) {
				private static final long serialVersionUID = 1L;

				@Override
				public void refreshResource() {

				}

			};

			fieldNorm = northPanel.getFormtedTextFieldInstance_Numeric();
			fieldNorm.setValue(2);
			fieldNorm.setColumns(3);
			fieldLimit = northPanel.getFormtedTextFieldInstance_Integer();
			fieldLimit.setValue(2);
			fieldLimit.setColumns(3);

			northPanel.addSimpleLabelAndInputRow(0,
					Dictionary.get(Bundle.METRIC, "quadraticnorm") + ":",
					fieldNorm);
			northPanel.addSimpleLabelAndInputRow(1,
					Dictionary.get(Bundle.METRIC, "quadraticlimit") + ":",
					fieldLimit);

			return northPanel;
		}

		@Override
		public HashMap<String, Object> getOptions() {
			HashMap<String, Object> map = new HashMap<String, Object>();
			map.put("norm", "" + fieldNorm.getValue());
			map.put("limit", "" + fieldLimit.getValue());
			return map;
		}
	}

}
